Components.utils.import("resource://gre/modules/Services.jsm");
var utf8_Converter = Components.classes["@mozilla.org/intl/utf8converterservice;1"].getService(Components.interfaces.nsIUTF8ConverterService);
var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
var gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);

// The lifetime of the three global variables below should be the same

var httpObserver_first_observe = false;
var arr_filters = [];
var arr_filters_unicode = [];

function get_Operating_System() {
    var os = Services.appinfo.OS.toLowerCase();
	
	if (os.startsWith("win")) {
		return "Windows";
		
	} else if (os == "linux") {
		return "Linux";
		
	} else{
		return "";
	}
}

function refresh_prefs__modify_HTTP_response(){
	var useragent_override_pref_Svc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.QueryInterface(Components.interfaces.nsIPrefBranch2 || Components.interfaces.nsIPrefBranch);

	useragent_override_pref_Svc.savePrefFile(null);
}

function parseReg(src) {
	var match = src.match(/^\/(.+)\/([igm]{0,3})$/);
	if (match) {
		return new RegExp(match[1],match[2]);
	} else {
		return src;
	}
}

function format_filter_string(str){
	var encoded = encodeURIComponent(str);

	// alert(encoded);		// %C4%83

	var decodedString = encoded.split('%').slice(1).map(u => String.fromCharCode(parseInt(u, 16))).join('');

	// alert(decodedString);		// Ä
	
	if(decodedString != ""){
		return decodedString;
		
	} else{
		return str;
	}
}

function get_filters_from__prefs_js(){
	var filter = "";
	var filter_unicode = "";
	
	try {
		filter = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("filter");
		
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.filter";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		filter_unicode = str.data;
		
	} catch(err){
		// alert(err.message);
		
		arr_filters.length = 0;
		arr_filters_unicode.length = 0;
		
		return false;
	}
	
	if(filter != ""){
		try {
			arr_filters = JSON.parse(filter);
			arr_filters_unicode = JSON.parse(filter_unicode);
			
			for (var i = 0; i < arr_filters.length; i++){
				arr_filters[i][0] = parseReg(arr_filters[i][0]);			// host
				arr_filters[i][1] = parseReg(arr_filters[i][1]);			// path
				
				arr_filters_unicode[i][0] = parseReg(arr_filters_unicode[i][0]);			// host formatted
				arr_filters_unicode[i][1] = parseReg(arr_filters_unicode[i][1]);			// path formatted
				
				for (var j = 2; j < arr_filters[i].length; j++){
					arr_filters[i][j][0] = parseReg(arr_filters[i][j][0]);	// search
				}
			}
			
			return true;
			
		} catch(err){
			// alert(err.message);		// JSON.parse: bad escaped character ...
		
			arr_filters.length = 0;
			arr_filters_unicode.length = 0;
			
			return false;
		}
		
	}else if(filter == ""){
		arr_filters.length = 0;
		arr_filters_unicode.length = 0;
		
		return true;
	}
}

var httpObserver = {
	observe: function(subject, topic, data) {
		if (topic == 'http-on-examine-response' || topic == 'http-on-examine-cached-response') {
			var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
			
			if(modify_response_OR_get_requests_info == "modify_response"){
				if( !httpObserver_first_observe ){
					var filters_from__prefs_js__OK = get_filters_from__prefs_js();
					
					if(filters_from__prefs_js__OK){
						httpObserver_first_observe = true;
						
					} else if( !filters_from__prefs_js__OK ){
						reset__modify_response__settings(filters_from__prefs_js__OK);
					}
				}
				
				if(arr_filters.length > 0){
					try{
						subject.QueryInterface(Components.interfaces.nsIHttpChannel);
							
						for (var i=0; i < arr_filters.length; i++) {
							
							var subject_URI_host = decodeURIComponent(subject.URI.host);
							var subject_URI_path = decodeURIComponent(subject.URI.path);
							
							if (typeof arr_filters_unicode[i][0] == "string" ? arr_filters_unicode[i][0] == subject_URI_host : subject_URI_host.search(arr_filters_unicode[i][0]) != -1) {
								if (typeof arr_filters_unicode[i][1] == "string" ? arr_filters_unicode[i][1] == subject_URI_path : subject_URI_path.search(arr_filters_unicode[i][1]) != -1) {							
									subject.QueryInterface(Components.interfaces.nsITraceableChannel);
									
									var newListener = new TracingListener();
									newListener.action_type = "modify_response";
									newListener.filter_index = i;
									newListener.originalListener = subject.setNewListener(newListener);
									
									break;
								}
							}
						}
					} catch(err){
						// console.log('ERROR in httpObserver: ', err.message);
					}
				}
				
			} else if(modify_response_OR_get_requests_info == "get_info"){
				var get_info_option = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("get_info_option");
				
				try{
					subject.QueryInterface(Components.interfaces.nsIHttpChannel);
					
					if(get_info_option == "get_URLs_Hosts_Paths"){
						subject.QueryInterface(Components.interfaces.nsITraceableChannel);
						
						var newListener = new TracingListener();
						newListener.action_type = "get_URLs_Hosts_Paths";
						newListener.originalListener = subject.setNewListener(newListener);
						
					} else if(get_info_option == "get_HTTP_Responses"){
						subject.QueryInterface(Components.interfaces.nsITraceableChannel);
						
						var newListener = new TracingListener();
						newListener.action_type = "get_HTTP_Responses";
						newListener.originalListener = subject.setNewListener(newListener);
						
					} else if(get_info_option == "get_Received_Data"){
						var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
						var branch_name = "extensions.modify_HTTP_response.host_for_get_data";
						var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
						var host_for_get_data = str.data;
						
						var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
						var branch_name = "extensions.modify_HTTP_response.path_for_get_data";
						var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
						var path_for_get_data = str.data;
						
						host_for_get_data = parseReg(host_for_get_data);
						path_for_get_data = parseReg(path_for_get_data);
						
						var subject_URI_host = decodeURIComponent(subject.URI.host);
						var subject_URI_path = decodeURIComponent(subject.URI.path);
						
						if (typeof host_for_get_data == "string" ? host_for_get_data == subject_URI_host : subject_URI_host.search(host_for_get_data) != -1) {
							if (typeof path_for_get_data == "string" ? path_for_get_data == subject_URI_path : subject_URI_path.search(path_for_get_data) != -1) {	
								subject.QueryInterface(Components.interfaces.nsITraceableChannel);
								
								var newListener = new TracingListener();
								newListener.action_type = "get_Received_Data";
								newListener.originalListener = subject.setNewListener(newListener);
							}
						}
					}
					
				} catch(err){
					// console.log('ERROR in httpObserver: ', err.message);
				}
			}
		}
	},
	QueryInterface: function(aIID) {
		if (aIID.equals(Components.interfaces.nsIObserver) || aIID.equals(Components.interfaces.nsISupports)) {
			return this;
		} else {
			throw Components.results.NS_NOINTERFACE;
		}
	},
	register: function() {
		try{
			Services.obs.addObserver(this, "http-on-examine-cached-response", false);
			Services.obs.addObserver(this, "http-on-examine-response", false);
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("httpObserver_registered", true);			
			httpObserver_first_observe = false;
			
			var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
			
			if(modify_response_OR_get_requests_info == "modify_response"){
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "");
				
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", "");
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
				
				document.getElementById("btn_modify_HTTP_response").setAttribute("image", "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_BLUE.png");
				
			} else if(modify_response_OR_get_requests_info == "get_info"){
				document.getElementById("btn_modify_HTTP_response").setAttribute("image", "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_PURPLE.png");
			}
			
			refresh_prefs__modify_HTTP_response();
			
		} catch(err){
			// console.log('ERROR in "register" : ', err.message);
		}
	},
	unregister: function(temp) {
		try{
			Services.obs.removeObserver(this, "http-on-examine-cached-response");
			Services.obs.removeObserver(this, "http-on-examine-response");
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("httpObserver_registered", false);
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "");
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", "");
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
			
			refresh_prefs__modify_HTTP_response();
			
			if(!temp){
				document.getElementById("btn_modify_HTTP_response").setAttribute("image", "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png");
			}
			
		} catch(err){
			// console.log('ERROR in "unregister" : ', err.message);
		}
	}
}

function write__requests_info__append_text(data_from_request){
	var data_from_request_unicode = utf8_Converter.convertURISpecToUTF8(data_from_request, "UTF-8");
	
	data_from_request_unicode = data_from_request_unicode + "\n\n";
	
	var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
	var ProfilesDir = directoryService.get("ProfD", Components.interfaces.nsIFile);
	var ProfilesDir_Path = ProfilesDir.path;
	
	var requests_info__file_path = "";

	var operating_system = get_Operating_System();

	if(operating_system == "Windows"){
		ProfilesDir_Path = ProfilesDir_Path.replace(/\\/g,'\\');	
		requests_info__file_path = ProfilesDir_Path + "\\modify_HTTP_response\\requests_info.txt";
		
	} else if(operating_system == "Linux"){
		requests_info__file_path = ProfilesDir_Path + "/modify_HTTP_response/requests_info.txt";
	}
	
	var requests_info__file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	requests_info__file.initWithPath(requests_info__file_path);
	
	// "requests_info__file" is nsIFile, "data_from_request_unicode" is a string
	
	var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

	// fileOutputStream.init(requests_info__file, 0x02 | 0x08 | 0x20, 0666, 0); 
	
	// use 0x02 | 0x10 to open file for appending
	// fileOutputStream.init(requests_info__file, 0x02 | 0x10, 0666, 0);
	
	fileOutputStream.init(requests_info__file, 0x02 | 0x10, 0666, 0);

	var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
	converter.init(fileOutputStream, "UTF-8", 0, 0);
	converter.writeString(data_from_request_unicode);
	fileOutputStream.close();
	converter.close();
	
	return;
}
	
function CCIN(cName, ifaceName) {
   	return Components.classes[cName].createInstance(Components.interfaces[ifaceName]);
}

function TracingListener() {
	this.receivedData = [];
}
	
TracingListener.prototype = {
	onDataAvailable: function(request, context, inputStream, offset, count) {
		var binaryInputStream = CCIN("@mozilla.org/binaryinputstream;1","nsIBinaryInputStream");
		binaryInputStream.setInputStream(inputStream);
		var data = binaryInputStream.readBytes(count);
		this.receivedData.push(data);
	},
	onStartRequest: function(request, context) {
		try {
			this.originalListener.onStartRequest(request, context);
		} catch (err) {
			request.cancel(err.result);
		}
	},
	onStopRequest: function(request, context, statusCode) {
		var data = this.receivedData.join("");

		if(this.action_type == "modify_response"){
			try {
				for (var j = 2; j < arr_filters[this.filter_index].length; j++){
					data = data.replace(arr_filters[this.filter_index][j][0], arr_filters[this.filter_index][j][1]);
				}
				
			} catch(err){
				// console.log('ERROR in onStopRequest: ', err.message);
			}
			
		} else if(this.action_type == "get_URLs_Hosts_Paths"){
			try{
				console.log('request.URI.spec : ', request.URI.spec);
				write__requests_info__append_text('request.URI.spec : ' + request.URI.spec);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.host : ', request.URI.host);
				write__requests_info__append_text('request.URI.host : ' + request.URI.host);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.path : ', request.URI.path);
				write__requests_info__append_text('request.URI.path : ' + request.URI.path);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('\n\n\nNext "request" !!!\n\n\n');
				write__requests_info__append_text('\n\nNext "request" !!!\n\n');
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
		} else if(this.action_type == "get_HTTP_Responses"){
			try{
				console.log('request.URI.spec : ', request.URI.spec);
				write__requests_info__append_text('request.URI.spec : ' + request.URI.spec);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.originalURI.spec : ', request.originalURI.spec);
				write__requests_info__append_text('request.originalURI.spec : ' + request.originalURI.spec);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.host : ', request.URI.host);
				write__requests_info__append_text('request.URI.host : ' + request.URI.host);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.path : ', request.URI.path);
				write__requests_info__append_text('request.URI.path : ' + request.URI.path);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}

			try{
				console.log('request.originalURI.path : ', request.originalURI.path);
				write__requests_info__append_text('request.originalURI.path : ' + request.originalURI.path);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.filePath : ', request.URI.filePath);
				write__requests_info__append_text('request.URI.filePath : ' + request.URI.filePath);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				var date = Date.parse(request.getResponseHeader("Date"));
				
				console.log('request.getResponseHeader("Date") : ', date);
				write__requests_info__append_text('request.getResponseHeader("Date") : ' + date);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.contentType : ', request.contentType);
				write__requests_info__append_text('request.contentType : ' + request.contentType);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.scheme : ', request.URI.scheme);
				write__requests_info__append_text('request.URI.scheme : ' + request.URI.scheme);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.URI.originCharset :', request.URI.originCharset);
				write__requests_info__append_text('request.URI.originCharset : ' + request.URI.originCharset);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('request.requestMethod : ', request.requestMethod);
				write__requests_info__append_text('request.requestMethod : ' + request.requestMethod);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('data : ', data);
				write__requests_info__append_text('data : ' + data);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('\n\n\nNext "request" !!!\n\n\n');
				write__requests_info__append_text('\n\nNext "request" !!!\n\n');
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
		} else if(this.action_type == "get_Received_Data"){
			try{
				console.log('request.URI.spec : ', request.URI.spec);
				write__requests_info__append_text('request.URI.spec : ' + request.URI.spec);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('data : ', data);
				write__requests_info__append_text('data : ' + data);
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
			
			try{
				console.log('\n\n\nNext "request" !!!\n\n\n');				
				write__requests_info__append_text('\n\nNext "request" !!!\n\n');
				
			} catch(err){
				// console.log("ERROR : ",  err.message);
			}
		}
		
		var storageStream = CCIN("@mozilla.org/storagestream;1", "nsIStorageStream");
		storageStream.init(8192, data.length, null);
		var os = storageStream.getOutputStream(0);
		if (data.length > 0) {
			os.write(data, data.length);
		}
		os.close();

		try {
			this.originalListener.onDataAvailable(request, context, storageStream.newInputStream(0), 0, data.length);
		} catch (e) {}

		try {
			this.originalListener.onStopRequest(request, context, statusCode);
		} catch (e) {}
	},
	QueryInterface: function(aIID) {
		if (aIID.equals(Components.interfaces.nsIStreamListener) || aIID.equals(Components.interfaces.nsISupports)) {
			return this;
		} else {
			throw Components.results.NS_NOINTERFACE;
		}
	}
}


function add_new_filter(arr_from_arr_filters, index){
	var host = "";
	var path = "";
	var search = "";
	var replace = "";
	
	for(var i=0; i<arr_from_arr_filters.length; i++){
		if(i == 0){
			// typeof arr_from_arr_filters[i] = string
			
			host = arr_from_arr_filters[i].trim();
			
		} else if(i == 1){
			// typeof arr_from_arr_filters[i] = string
			
			path = arr_from_arr_filters[i].trim();
			
		} else if(i == 2){
			// typeof arr_from_arr_filters[i] = object
			// arr_from_arr_filters[i].length = 2
			
			search = arr_from_arr_filters[i][0];
			replace = arr_from_arr_filters[i][1];
		}
	}

	var int_new_filter_index = index;

	var str_new_filter_innerHTML = '<div id="host_' + int_new_filter_index + '_tag" class="host_tag">host</div><div class="a_space">&nbsp;</div><textarea id="host_' + int_new_filter_index + '" class="host_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;">' + host + '</textarea><div class="a_space">&nbsp;</div><div id="path_' + int_new_filter_index + '_tag" class="path_tag">path</div><div class="a_space">&nbsp;</div><textarea id="path_' + int_new_filter_index + '" class="path_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;">' + path + '</textarea><br><div id="SearchReplace_' + int_new_filter_index + '_0" class="search_replace"><div id="search_' + int_new_filter_index + '_0_tag" class="search_tag">search</div><div class="a_space">&nbsp;</div><textarea id="search_' + int_new_filter_index + '_0" class="search_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;">' + search + '</textarea><div class="a_space">&nbsp;</div><div id="replace_' + int_new_filter_index + '_0_tag" class="replace_tag">replace</div><div class="a_space">&nbsp;</div><textarea id="replace_' + int_new_filter_index + '_0" class="replace_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;" class="search_replace">' + replace + '</textarea><br></div>';

	var obj_new_filter = content.document.createElement("div");
	var last_inserted_filter_id = "filter_" + int_new_filter_index;

	obj_new_filter.id = last_inserted_filter_id;
	obj_new_filter.className = "filter_div";		
	obj_new_filter.innerHTML = str_new_filter_innerHTML;

	var filters_div = content.document.getElementById("filters_div");

	filters_div.appendChild(obj_new_filter);

	var new_host_id = "host_" + int_new_filter_index;

	content.document.getElementById(new_host_id).focus();
	
	for(var j=3; j<arr_from_arr_filters.length; j++){
		var str_search = arr_from_arr_filters[j][0];
		var str_replace = arr_from_arr_filters[j][1];
		
		add_new_search_replace(last_inserted_filter_id, str_search, str_replace);
	}
}

function add_new_search_replace(last_inserted_filter_id, search, replace){
	var obj_new_filter = content.document.getElementById(last_inserted_filter_id);
	var int_focused_filter_index = last_inserted_filter_id.split("_")[1];

	var search_tag_length = obj_new_filter.getElementsByClassName("search_tag").length;
	var search_replace_index = search_tag_length;

	var str_new_search_replace_innerHTML = '<div id="search_' + int_focused_filter_index + '_' + search_replace_index + '_tag" class="search_tag">search</div><div class="a_space">&nbsp;</div><textarea id="search_' + int_focused_filter_index + '_' + search_replace_index + '" class="search_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;">' + search + '</textarea><div class="a_space">&nbsp;</div><div id="replace_' + int_focused_filter_index + '_' + search_replace_index + '_tag" class="replace_tag">replace</div><div class="a_space">&nbsp;</div><textarea id="replace_' + int_focused_filter_index + '_' + search_replace_index + '" class="replace_textarea" rows="1" spellcheck="false" onBlur="save_last_focused_textarea_id(this.id); return false;">' + replace + '</textarea><br>';

	var obj_focused_filter = content.document.getElementById(last_inserted_filter_id);

	var obj_new_search_replace = content.document.createElement("div");
	obj_new_search_replace.id = "SearchReplace_" + int_focused_filter_index + "_" + search_replace_index;
	obj_new_search_replace.className = "search_replace";
	obj_new_search_replace.innerHTML = str_new_search_replace_innerHTML;

	obj_focused_filter.appendChild(obj_new_search_replace);

	var str_new_search_id = "search_" + int_focused_filter_index + "_" + search_replace_index;

	content.document.getElementById(str_new_search_id).focus();
}

function fill__filters_manager__with_filters_from__prefs(){
	var filter = "";

	try{
		// filter = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("filter");
		
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.filter";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		filter = str.data;
		
	} catch(err){
		// alert(err.message);
	}
	
	var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
	var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
	
	var title = content.document.getElementsByTagName("title")[0];
	var favicon = content.document.createElement("link");
	favicon.type = "image/x-icon";
	favicon.rel = "shortcut icon";
	
	if(modify_response_OR_get_requests_info == "modify_response" && httpObserver_registered){
		favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_BLUE.png";
	
	} else if(modify_response_OR_get_requests_info == "get_info" || !httpObserver_registered){
		favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png";
	}
	
	title.parentNode.insertBefore(favicon, title.nextSibling);

	if(filter != ""){
		if(modify_response_OR_get_requests_info == "modify_response" && httpObserver_registered){
			content.document.getElementById("httpObserver_registered").checked = true;
			content.document.getElementById("httpObserver_registered_status").value = "true";

		
		} else if(modify_response_OR_get_requests_info == "get_info" || !httpObserver_registered){
			content.document.getElementById("httpObserver_registered").checked = false;
			content.document.getElementById("httpObserver_registered_status").value = "false";
		}
		
		var arr_filters__for_filters_manager = JSON.parse(filter);

		for(var i=0; i<arr_filters__for_filters_manager.length; i++){
			var arr_from_arr_filters = arr_filters__for_filters_manager[i];
			
			add_new_filter(arr_from_arr_filters, i);
		}
		
	} else{
		content.document.getElementById("httpObserver_registered").checked = false;
		content.document.getElementById("httpObserver_registered_status").value = "false";
			
		prompts.alert(null, 'Filters', 'There are no Filters Saved in the "prefs.js" file');
	}
}

function fill__requests_info__with__url_host_path__from__prefs(){
	var url_page = "";

	try{
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.url_page";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		url_page = str.data;
		
	} catch (err) {
		// alert(err.message);
	}
	
	if(url_page != ""){
		content.document.getElementById("url_page").value = url_page;
	}
	
	var host_for_get_data = "";
	var path_for_get_data =  "";
	
	try{
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.host_for_get_data";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		host_for_get_data = str.data;
		
	} catch (err) {
		// alert(err.message);
	}
	
	try{
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.path_for_get_data";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		path_for_get_data = str.data;
		
	} catch (err) {
		// alert(err.message);
	}
	
	if(host_for_get_data != "" && path_for_get_data != ""){
		content.document.getElementById("single_host").value = host_for_get_data;
		content.document.getElementById("single_path").value = path_for_get_data;
	}
}

function btn_modify_HTTP_response__onclick(event){
	if(event.button == 0){
		
		// "Click" on Button = Open "Filters Manager"
		
		if(!event.ctrlKey && !event.altKey && !event.shiftKey){
			var filters_from__prefs_js__OK = get_filters_from__prefs_js();

			if(filters_from__prefs_js__OK){
				var mrw = Services.wm.getMostRecentWindow('navigator:browser');
				
				if(typeof mrw.gBrowser=='undefined'){
					mrw.BrowserApp.selectOrAddTab('chrome://modify_HTTP_response/content/filters_manager.html');
					
					setTimeout(function() {
						fill__filters_manager__with_filters_from__prefs();
						
					}, 1000);
				
				} else{
					mrw.gBrowser.loadOneTab('chrome://modify_HTTP_response/content/filters_manager.html',{inBackground:false,relatedToCurrent:true});
					
					setTimeout(function() {
						fill__filters_manager__with_filters_from__prefs();
						
					}, 1000);
				}
				
			} else if( !filters_from__prefs_js__OK ){
				reset__modify_response__settings(filters_from__prefs_js__OK);
			}
		}
		
		// "Ctrl + Click" on Button = register "httpObserver"
		
		else if(event.ctrlKey && !event.altKey && !event.shiftKey){
			var filters_from__prefs_js__OK = get_filters_from__prefs_js();

			if(filters_from__prefs_js__OK){
				if(arr_filters.length > 0){
					var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
					
					if(httpObserver_registered){
						var temp = true;
						httpObserver.unregister(temp);
					}
					
					Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
					
					refresh_prefs__modify_HTTP_response();
					
					setTimeout(function() {
						httpObserver.register();
						
					}, 100);
					
				} else{
					prompts.alert(null, 'Register "HTTP Observer"', '"HTTP Observer" was NOT enabled because there are NO Saved Filters in "prefs.js".');
				}
				
			} else if( !filters_from__prefs_js__OK ){
				reset__modify_response__settings(filters_from__prefs_js__OK);
			}
		}
		
		// "Alt + Click" on Button = unregister "httpObserver"
		
		else if (event.altKey && !event.ctrlKey && !event.shiftKey){
			var filters_from__prefs_js__OK = get_filters_from__prefs_js();

			if(filters_from__prefs_js__OK){
				var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
				
				if(httpObserver_registered){
					var temp = false;
					httpObserver.unregister(temp);
				}
				
			} else if( !filters_from__prefs_js__OK ){
				reset__modify_response__settings(filters_from__prefs_js__OK);
			}
		}
		
		// "Shift + Click" on Button = Open "Requests Info"
		
		else if(event.shiftKey && !event.ctrlKey && !event.altKey){
			var filters_from__prefs_js__OK = get_filters_from__prefs_js();

			if(filters_from__prefs_js__OK){
				var mrw = Services.wm.getMostRecentWindow('navigator:browser');
				
				if(typeof mrw.gBrowser=='undefined'){
					mrw.BrowserApp.selectOrAddTab('chrome://modify_HTTP_response/content/requests_info.html');
				
				} else{
					mrw.gBrowser.loadOneTab('chrome://modify_HTTP_response/content/requests_info.html',{inBackground:false,relatedToCurrent:true});
				}
				
				setTimeout(function() {
					content.document.getElementById("url_page").focus();
					
					var title = content.document.getElementsByTagName("title")[0];
					var favicon = content.document.createElement("link");
					favicon.type = "image/x-icon";
					favicon.rel = "shortcut icon";
					
					var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
						
					if(modify_response_OR_get_requests_info == "get_info"){
						favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_PURPLE.png";				
						title.parentNode.insertBefore(favicon, title.nextSibling);
					
					} else if(modify_response_OR_get_requests_info == "modify_response"){
						favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_GREY.png";				
						title.parentNode.insertBefore(favicon, title.nextSibling);
					}
					
					fill__requests_info__with__url_host_path__from__prefs();
					
				}, 1000);
				
			} else if( !filters_from__prefs_js__OK ){
				reset__modify_response__settings(filters_from__prefs_js__OK);
			}
		}
		
		// "Ctrl + Shift + Click" on Button = Open Folder with "requests_info.txt"
		
		else if(event.ctrlKey && event.shiftKey && !event.altKey){
			var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);

			var ProfilesDir = directoryService.get("ProfD", Components.interfaces.nsIFile);
			var ProfilesDir_Path = ProfilesDir.path;
			
			var requests_info__folder_path = "";

			var operating_system = get_Operating_System();

			if(operating_system == "Windows"){
				ProfilesDir_Path = ProfilesDir_Path.replace(/\\/g,'\\');	
				requests_info__folder_path = ProfilesDir_Path + '\\modify_HTTP_response';
				
			} else if(operating_system == "Linux"){
				requests_info__folder_path = ProfilesDir_Path + "/modify_HTTP_response";
			}

			var localFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

			localFile.initWithPath(requests_info__folder_path);

			localFile.reveal();
		}
		
		// "Ctrl + Alt + Click" on Button = Open "Browser Console" OR Focus ( if already opened )
		
		else if(event.ctrlKey && event.altKey && !event.shiftKey){
			var {devtools} = Components.utils.import("resource://gre/modules/devtools/Loader.jsm", {});
			var {HUDService} = devtools.require("devtools/client/webconsole/hudservice");

			if(HUDService.getBrowserConsole()){ 	// If it is already opened
				HUDService.getBrowserConsole().chromeWindow.focus();
				
			} else{
				HUDService.toggleBrowserConsole().chromeWindow;
			}
		}
		
		// "Ctrl + Alt + Shift + Click" on Button = TEST
		
		else if(event.ctrlKey && event.altKey && event.shiftKey){
			modify_HTTP_response__TEST();
		}
	}
}

function modify_HTTP_response__TEST(){
	try{
		var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
		
		alert('Variable "httpObserver_registered" - value from "prefs.js" : ' + httpObserver_registered);		
		alert('Global variable "httpObserver_first_observe" : ' + httpObserver_first_observe);
		
	} catch(err){
		alert('ERROR in "modify_HTTP_response__TEST" : ' + err.message);
	}
	
	try{
		alert('Global variable "arr_filters" : ' + arr_filters);
		
	} catch(err){
		alert('ERROR in "modify_HTTP_response__TEST" : ' + err.message);
	}
	
	try{
		alert('Global variable "arr_filters_unicode" : ' + arr_filters_unicode);
		
	} catch(err){
		alert('ERROR in "modify_HTTP_response__TEST" : ' + err.message);
	}
}

function modify_HTTP_response__menupopup__right_click() {
	var mPopup = document.getElementById("modify_HTTP_response__menupopup");
	
	if(!mPopup){
		return;
	}
	
	while (mPopup.hasChildNodes()) {
		mPopup.removeChild(mPopup.firstChild);
	}

	var icon_LIST_HEADER = "chrome://modify_HTTP_response/content/modify_HTTP_response_LIST_HEADER.png";
	var icon_LIST = "chrome://modify_HTTP_response/content/modify_HTTP_response_LIST.png";
	
	var menuitem = [];
	
	var label_HEADER = 'Delete, Backup or Restore "Filters" :';
	
	var label__delete_all_filters = 'Delete All Filters';
	var value__delete_all_filters = 'delete';
	
	var label__backup_filters = 'Backup Filters ( Copy to Clipboard )';
	var value__backup_filters = 'backup';
	
	var label__restore_all_filters = 'Restore Filters From Backup ( Replaces Existing Filters ! )';
	var value__restore_all_filters = 'restore_all';
	
	var label__restore_one_filter = 'Add Filters From Backup ( To Current Filters )';
	var value__restore_one_filter = 'restore_one';
	
	var label__reset = 'RESET ! - Initialize Settings';
	var value__reset = 'reset';
	
	var label__convert = 'Convert Filter Syntax from "Modify HTTP Response" format to this Add-On format';
	var value__convert = 'convert';
	
	// Add "menuseparator"
	
	menuitem[0] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[1] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[1].setAttribute("label", label_HEADER);
	menuitem[1].setAttribute("value", "");
	menuitem[1].setAttribute("id", "menuitem__modify_http_response");
	menuitem[1].setAttribute("class", "menuitem-iconic");
	menuitem[1].setAttribute("image", icon_LIST_HEADER);
	menuitem[1].setAttribute("onmouseenter", "event.preventDefault();");
	menuitem[1].setAttribute("onmousemove", "event.preventDefault();");
	menuitem[1].setAttribute("onmouseout", "event.preventDefault();");
	menuitem[1].setAttribute("onmouseover", "event.preventDefault();");
	menuitem[1].setAttribute("oncommand", "event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[2] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[3] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[3].setAttribute("label", label__delete_all_filters);
	menuitem[3].setAttribute("value", value__delete_all_filters);
	menuitem[3].setAttribute("id", "menuitem__delete_all_filters");
	menuitem[3].setAttribute("class", "menuitem-iconic");
	menuitem[3].setAttribute("image", icon_LIST);
	menuitem[3].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[4] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[5] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[5].setAttribute("label", label__backup_filters);
	menuitem[5].setAttribute("value", value__backup_filters);
	menuitem[5].setAttribute("id", "menuitem__backup_filters");
	menuitem[5].setAttribute("class", "menuitem-iconic");
	menuitem[5].setAttribute("image", icon_LIST);
	menuitem[5].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[6] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[7] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[7].setAttribute("label", label__restore_all_filters);
	menuitem[7].setAttribute("value", value__restore_all_filters);
	menuitem[7].setAttribute("id", "menuitem__restore_all_filters");
	menuitem[7].setAttribute("class", "menuitem-iconic");
	menuitem[7].setAttribute("image", icon_LIST);
	menuitem[7].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[8] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[9] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[9].setAttribute("label", label__restore_one_filter);
	menuitem[9].setAttribute("value", value__restore_one_filter);
	menuitem[9].setAttribute("id", "menuitem__restore_one_filter");
	menuitem[9].setAttribute("class", "menuitem-iconic");
	menuitem[9].setAttribute("image", icon_LIST);
	menuitem[9].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[10] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[11] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[11].setAttribute("label", label__reset);
	menuitem[11].setAttribute("value", value__reset);
	menuitem[11].setAttribute("id", "menuitem__reset");
	menuitem[11].setAttribute("class", "menuitem-iconic");
	menuitem[11].setAttribute("image", icon_LIST);
	menuitem[11].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[12] = mPopup.appendChild(document.createElement("menuseparator"));
	
	// Add "item"
	
	menuitem[13] = mPopup.appendChild(document.createElement("menuitem"));
	menuitem[13].setAttribute("label", label__convert);
	menuitem[13].setAttribute("value", value__convert);
	menuitem[13].setAttribute("id", "menuitem__convert");
	menuitem[13].setAttribute("class", "menuitem-iconic");
	menuitem[13].setAttribute("image", icon_LIST);
	menuitem[13].setAttribute("oncommand", "modify_HTTP_response__menupopup__manage_right_click_events(event); event.stopPropagation();");
	
	// Add "menuseparator"
	
	menuitem[14] = mPopup.appendChild(document.createElement("menuseparator"));
}

function modify_HTTP_response__menupopup__manage_right_click_events(event) {
	var filters_from__prefs_js__OK = get_filters_from__prefs_js();

	if(filters_from__prefs_js__OK){
		var str_value = event.target.getAttribute("value");
		
		if(str_value == 'delete'){
			delete_all_filters();
			
		} else if(str_value == 'backup'){
			backup_all_filters();
			
		} else if(str_value == 'restore_all'){
			restore_all_filters();
			
		} else if(str_value == 'restore_one'){
			add_only_one_filter();
			
		} else if(str_value == 'reset'){
			reset__modify_response__settings(filters_from__prefs_js__OK);
			
		} else if(str_value == 'convert'){
			get_filter_with__Modify_HTTP_Response__syntax();
		}
		
	} else if( !filters_from__prefs_js__OK ){
		reset__modify_response__settings(filters_from__prefs_js__OK);
	}
}

function delete_all_filters(){
	var btn_OK = prompts.confirm(null, 'Delete All Filters', ' \n Do you want to Delete All Filters ? \n ');
	
	if (!btn_OK){
		return;
	
	} else{
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", "");
		
		httpObserver_first_observe = false;

		refresh_prefs__modify_HTTP_response();
		
		arr_filters.length = 0;
		arr_filters_unicode.length = 0;
		
		var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
		
		if(httpObserver_registered){
			var temp = false;
			httpObserver.unregister(temp);
		}

		prompts.alert(null, 'Delete All Filters', 'All Filters have been Removed from the "prefs.js" file');
	}
}

function backup_all_filters(){
	var filter = "";

	try{
		// filter = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("filter");
		
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.filter";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		filter = str.data;
		
	} catch(err){
		// alert(err.message);
		
		prompts.alert(null, 'Backup All Filters', 'ERROR : Filters saved in "prefs.js" file are corrupted.');
		
		return;
	}

	if(filter != ""){
		try{
			// [host, path, [search, replace]]
			
			// [`stackoverflow.com`, `/`, [`/&/g`, ``]]
			// [host, path, [search, __ZERO_LENGTH_STRING__REPLACE__]]

			filter = filter.replace(/(\[[^\[]*?`[\s]*,[\s]*`)(`[\s]*\])/g, '$1__ZERO_LENGTH_STRING__REPLACE__`]');
			filter = filter.replace(/`/g, '__backticks__');
			
			var arr_filters = JSON.parse(filter);

			var str_filters__for_Backup = "";
			
			var arr_filters_length = arr_filters.length;

			for(var i=0; i<arr_filters.length; i++){
				var arr_arrays = arr_filters[i];
				
				var str_filter = "";
				
				for(var j=0; j<arr_arrays.length; j++){
					if(typeof arr_arrays[j] == "string"){
						if(j == 0){
							str_filter = "[`" + arr_arrays[j] + "`, ";
							
						} else if(j == 1){
							str_filter = str_filter + "`" +  arr_arrays[j] + "`";
						}
					
					} else if(typeof arr_arrays[j] == "object"){
						if(j > 1){
							str_filter = str_filter + ", [`" + arr_arrays[j][0] + "`, `" + arr_arrays[j][1] + "`]";
						}
					}
				}
				
				str_filter = str_filter + "]";
				
				if(i == 0){
					if(arr_filters_length == 1){							// There is only one array in filters
						str_filters__for_Backup = str_filter;
						
					} else if(arr_filters_length > 1){
						str_filters__for_Backup = "[" + str_filter;
					}
				
				} else if(i == arr_filters.length - 1){
					str_filters__for_Backup = str_filters__for_Backup + ", " + str_filter + "]";
					
				} else{
					str_filters__for_Backup = str_filters__for_Backup + ", " + str_filter;
				}
			}
			
			str_filters__for_Backup = str_filters__for_Backup.replace(/__ZERO_LENGTH_STRING__REPLACE__/g, '');
			str_filters__for_Backup = str_filters__for_Backup.replace(/__backticks__/g, '``');

			gClipboardHelper.copyString(str_filters__for_Backup);

			prompts.alert(null, 'Backup Filters', 'All Filters have been Copied to Clipboard.');
			
		} catch(err){
			// alert(err.message);
			
			prompts.alert(null, 'Backup All Filters', 'ERROR : Filters saved in "prefs.js" file are corrupted.');
		}
		
	} else if(filter == ""){
		prompts.alert(null, 'Backup Filters', 'There are no Filters Saved in the "prefs.js" file');
	}
}

function check_filter_syntax(str_filter){
	// [host, path, [search, replace]]
	
	// [`stackoverflow.com`, `/`, [`/&/g`, ``]]
	// [host, path, [search, __ZERO_LENGTH_STRING__REPLACE__]]
	
	str_filter = str_filter.replace(/(\[[^\[]*?`[\s]*,[\s]*`)(`[\s]*\])/g, '$1__ZERO_LENGTH_STRING__REPLACE__`]');
	str_filter = str_filter.replace(/``/g, '__backticks__');
	
	var str_filter__without_spaces = str_filter.replace(/\s/g, '');
	
	var first_2_characters_in_filter = str_filter__without_spaces.substr(0, 2);
	var last_3_characters_in_filter = str_filter__without_spaces.substring(str_filter__without_spaces.length - 3);
	
	var first_3_characters_in_filter = str_filter__without_spaces.substr(0, 3);
	var last_4_characters_in_filter = str_filter__without_spaces.substring(str_filter__without_spaces.length - 4);
	
	if(first_2_characters_in_filter == "[`"){
		if(last_3_characters_in_filter != "`]]"){
			return false;
		}
		
	} else if(first_3_characters_in_filter == "[[`"){
		if(last_4_characters_in_filter != "`]]]"){
			return false;
		}
		
	} else{
		return false;
	}
	
	var arr__arrays_structure_components__arrays_values__WITHOUT_SPACES = str_filter__without_spaces.split("`");
	
	var previous_character_group = "";
	
	for(var j=0; j<arr__arrays_structure_components__arrays_values__WITHOUT_SPACES.length; j++){
		try{
			// j is even = ONLY the 3 characters [ , ] that structure the array(s),
			// NOT the values of "host", "path", "search" and "replace" ( which are obtained with odd j, BUT we don't need them )
			
			if(j % 2 == 0){
				var str_temp = arr__arrays_structure_components__arrays_values__WITHOUT_SPACES[j];
				
				// j = multiple of 4 ( without j = 0 and without the last j - analyzed above ) = Characters that structure the array ( arrays ),
				// in various COMBINATIONS of 2 or 3 - NOT a single character
				
				if((j != 0) && (j != arr__arrays_structure_components__arrays_values__WITHOUT_SPACES.length - 1) && (j % 4 == 0)){
					if(j == 4){
						if(str_temp != ",["){
							return false;
						}
						
					} else if(previous_character_group == ",["){
						if((str_temp != "],[") && (str_temp != "]],[")){
							return false;
						}
						
					} else if(previous_character_group == "],["){
						if((str_temp != "],[") && (str_temp != "]],[")){
							return false;
						}
						
					} else if(previous_character_group == "]],["){
						if(str_temp != ",["){
							return false;
						}
					}
					
					previous_character_group = str_temp;
				}
				
				// j : 4 = x rest 2		( 2, 6, 10, 14, ETC. ) 
				// ONLY the character , ALONE
				// The character , belongs to the group of the 3 characters [ , ] that structure the array(s)
				
				if(j % 4 == 2){
					if(str_temp != ","){
						return false;
					}
				}
			}
			
		} catch(err){
			// alert(err.message);
		}
	}
	
	return true;
}

function restore_all_filters(){
	var check = {value: false};		// default the checkbox to false
	var input = {value: ""};		// default text = ""
	var return_value = prompts.prompt(null, "Restore Filters From Backup", 'Enter Previously Saved Filters ( Backup ) : ', input, null, check);

	var str_previously_saved_filters = input.value.trim();

	// Return value ("return_value") = "true" for OK button • "false" for Cancel button.

	if (return_value && str_previously_saved_filters != "") {
		var filter_syntax_OK = check_filter_syntax(str_previously_saved_filters);
		
		// Correct Syntax
		
		if(filter_syntax_OK){
			var title = 'Restore Filters From Backup';
			var msg = ' \n WARNING ! \n\n All Existing Filters will be Deleted, \n and will be Replaced with Filters from Backup ! \n\n Do you want to continue ? \n ';
			var checkbox_initial_state = {value: false};	// The checkbox will be hidden (object with its 'value' property set to a boolean)

			var flags = prompts.BUTTON_POS_0 * prompts.BUTTON_TITLE_YES + prompts.BUTTON_POS_1 * prompts.BUTTON_TITLE_NO; 

			// This value of flags will create 2 buttons. The first will be "Yes" and the second will be "No".

			var button_index = prompts.confirmEx(null, title, msg, flags, '', '', '', 'Register "HTTP Observer"', checkbox_initial_state);

			// Return value = Index of the button pressed (0 = Yes, 1 = No)
			// checkbox_initial_state.value is true if the box was checked and false if the box was cleared
				
			if (button_index == 1) {
				return;
				
			} else if (button_index == 0) {
				// [host, path, [search, replace]]
				
				// [`stackoverflow.com`, `/`, [`/&/g`, ``]]
				// [host, path, [search, __ZERO_LENGTH_STRING__REPLACE__]]
				
				str_previously_saved_filters = str_previously_saved_filters.replace(/(\[[^\[]*?`[\s]*,[\s]*`)(`[\s]*\])/g, '$1__ZERO_LENGTH_STRING__REPLACE__`]');
				str_previously_saved_filters = str_previously_saved_filters.replace(/``/g, '__backticks__');
				
				var str_filters__from_Backup = JSON.stringify(str_previously_saved_filters);
				str_filters__from_Backup = str_filters__from_Backup.replace(/`/g, '"');
				str_filters__from_Backup = str_filters__from_Backup.trim();
				str_filters__from_Backup = str_filters__from_Backup.substr(1, (str_filters__from_Backup.length - 2));
				
				var str_filters__from_Backup__without_spaces = str_filters__from_Backup.replace(/\s/g, '');
				var last_three_characters_in_filter = str_filters__from_Backup__without_spaces.substring(str_filters__from_Backup__without_spaces.length - 3);
				
				if(last_three_characters_in_filter != "]]]"){							// There is only one array in filters
					str_filters__from_Backup = "[" + str_filters__from_Backup + "]";
				}
				
				str_filters__from_Backup = str_filters__from_Backup.replace(/__ZERO_LENGTH_STRING__REPLACE__/g, '');
				str_filters__from_Backup = str_filters__from_Backup.replace(/__backticks__/g, '`');
				
				// Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_filters__from_Backup);
				
				var str_formated_filter_string = "";

				for(var i=0; i < str_filters__from_Backup.length; i++){
					str_formated_filter_string = str_formated_filter_string + format_filter_string(str_filters__from_Backup[i]);
				}

				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_formated_filter_string);
				
				httpObserver_first_observe = false;

				refresh_prefs__modify_HTTP_response();
				
				var register_httpObserver = checkbox_initial_state.value;
				
				if(register_httpObserver){
					var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
					
					if(httpObserver_registered){
						var temp = true;
						httpObserver.unregister(temp);
					}
					
					setTimeout(function() {
						Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
						
						refresh_prefs__modify_HTTP_response();
						
						setTimeout(function() {
							httpObserver.register();
							
							prompts.alert(null, 'Restore Filters From Backup', 'The Filters from Backup have been saved in the "prefs.js" file ');
							
						}, 100);
						
					}, 100);
					
				} else{
					prompts.alert(null, 'Restore Filters From Backup', 'The Filters from Backup have been saved in the "prefs.js" file ');
				}
			}
			
		}
		
		// Syntax with Errors
		
		else{
			prompts.alert(null, 'Restore Filters From Backup', '\nERROR ! \n\nEntered Filters have Syntax with Errors ! \n ');
		}
	}
}

function add_only_one_filter(){
	var filter = "";

	try{
		// filter = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("filter");
		
		var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var branch_name = "extensions.modify_HTTP_response.filter";
		var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
		filter = str.data;
		
	} catch(err){
		// alert(err.message);
	}

	if(filter != ""){
		var check = {value: false};		// default the checkbox to false
		var input = {value: ""};		// default text = ""
		var return_value = prompts.prompt(null, "Add Filters From Backup", 'Enter the Filters to be Added ( to the existing ones ) in the "prefs.js" file :', input, null, check);

		var str_previously_saved_filters = input.value.trim();

		// Return value ("return_value") = "true" for OK button • "false" for Cancel button.

		if (return_value && str_previously_saved_filters != "") {
			var filter_syntax_OK = check_filter_syntax(str_previously_saved_filters);
			
			// Correct Syntax
			
			if(filter_syntax_OK){
				var str_previously_saved_filters__without_spaces = str_previously_saved_filters.replace(/\s/g, '');
				var last_three_characters_in_filter = str_previously_saved_filters__without_spaces.substring(str_previously_saved_filters__without_spaces.length - 3);
				
				if(last_three_characters_in_filter == "]]]"){	// There are multiple arrays in filters OR a single array with multiple arrat syntax
					str_previously_saved_filters = str_previously_saved_filters.substr(1, (str_previously_saved_filters.length - 2));
				}
				
				// [host, path, [search, replace]]
				
				// [`stackoverflow.com`, `/`, [`/&/g`, ``]]
				// [host, path, [search, __ZERO_LENGTH_STRING__REPLACE__]]
			
				filter = filter.replace(/(\[[^\[]*?`[\s]*,[\s]*`)(`[\s]*\])/g, '$1__ZERO_LENGTH_STRING__REPLACE__`]');
				filter = filter.replace(/`/g, '__backticks__');
				
				str_previously_saved_filters = str_previously_saved_filters.replace(/(\[[^\[]*?`[\s]*,[\s]*`)(`[\s]*\])/g, '$1__ZERO_LENGTH_STRING__REPLACE__`]');
				str_previously_saved_filters = str_previously_saved_filters.replace(/``/g, '__backticks__');
				
				var arr_saved_filters = JSON.parse(filter);

				var str_current_filters = "";

				for(var i=0; i<arr_saved_filters.length; i++){
					var arr_arrays = arr_saved_filters[i];
					
					var str_filter = "";
					
					for(var j=0; j<arr_arrays.length; j++){
						if(typeof arr_arrays[j] == "string"){
							if(j == 0){
								str_filter = "[`" + arr_arrays[j] + "`, ";
								
							} else if(j == 1){
								str_filter = str_filter + "`" +  arr_arrays[j] + "`";
							}
						
						} else if(typeof arr_arrays[j] == "object"){
							if(j > 1){
								str_filter = str_filter + ", [`" + arr_arrays[j][0] + "`, `" + arr_arrays[j][1] + "`]";
							}
						}
					}
					
					str_filter = str_filter + "]";
					
					if(i == 0){
						str_current_filters = "[" + str_filter;
					
					} else{
						str_current_filters = str_current_filters + ", " + str_filter;
					}
				}
				
				var str_new_filters = str_current_filters + ", " + str_previously_saved_filters + "]";

				var str_filters = JSON.stringify(str_new_filters);
				str_filters = str_filters.replace(/`/g, '"');
				str_filters = str_filters.trim();
				var str_all_filters = str_filters.substr(1, (str_filters.length - 2));
				
				str_all_filters = str_all_filters.replace(/__ZERO_LENGTH_STRING__REPLACE__/g, '');
				str_all_filters = str_all_filters.replace(/__backticks__/g, '`');
				
				// Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_all_filters);
				
				var str_formated_filter_string = "";

				for(var i=0; i < str_all_filters.length; i++){
					str_formated_filter_string = str_formated_filter_string + format_filter_string(str_all_filters[i]);
				}

				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_formated_filter_string);
				
				httpObserver_first_observe = false;

				refresh_prefs__modify_HTTP_response();
				
				prompts.alert(null, 'Add Filters From Backup', 'The Filters from Backup have been Added ( to the existing ones ) in the "prefs.js" file');
		
			}
			
			// Syntax with Errors
			
			else{
				prompts.alert(null, 'Add Filters From Backup', '\nERROR ! \n\nEntered Filters have Syntax with Errors ! \n ');
			}
		}
		
	} else{
		prompts.alert(null, 'Add Filters From Backup', 'There are no Filters Saved in the "prefs.js" file. \nTo Restore ( NOT Add ) Filters, use the "Restore Filters From Backup" option. ');
	}
}

function reset__modify_response__settings(filters_from__prefs_js__OK){
	if(filters_from__prefs_js__OK){
		var title = 'Reset Add-On';
		var msg = ' \n WARNING ! \n\n All Existing Filters will be Deleted ! \n\n Do you want to Continue to Reset the Add-On ? \n ';	
			
		var btn_OK = prompts.confirm(null, title, msg);
		
		if (!btn_OK){
			return;
		
		} else{
			reset_settings_in__prefs_js();
		}
		
	} else if( !filters_from__prefs_js__OK ){
		var title = 'ERROR in "Intercept & Modify HTTP Response" Add-On !';
		var msg = '\nERROR : Filters saved in "prefs.js" file are corrupted ! \nThe add-on needs to be reset. \n\nWARNING : All Existing ( corrupted ) Filters will be Deleted ! \n"Backup Filters ( Copy to Clipboard )" option no longer works ! \nTo backup corrupted filters, you need to manually copy them from the "prefs.js" file. \n\nDo you want to Continue to Reset the Add-On ? \n ';
	
		var btn_OK = prompts.confirm(null, title, msg);
		
		if (!btn_OK){
			return;
		
		} else{
			reset_settings_in__prefs_js();
		}
	}
}

function get_filter_with__Modify_HTTP_Response__syntax(){
	var check = {value: false};		// default the checkbox to false
	var input = {value: ""};		// default text = ""
	var return_value = prompts.prompt(null, 'Convert Filter Syntax', 'Enter Filter to be Converted from "Modify HTTP Response" format to this Add-On format :', input, null, check);

	var str_filter_to_be_converted = input.value.trim();

	// Return value ("return_value") = "true" for OK button • "false" for Cancel button.

	if (return_value && str_filter_to_be_converted != "") {
		var converted_filter = convert_filters_syntax_from__Modify_HTTP_Response__to__Intercept_and_Modify_HTTP_Response(str_filter_to_be_converted);
		
		if(converted_filter == "ERROR"){
			prompts.alert(null, 'Convert Filter', 'Entered Filter have Syntax with Errors !');
			
		} else{
			gClipboardHelper.copyString(converted_filter);
			
			prompts.alert(null, 'Convert Filter', 'Converted Filter has been copied to Clipboard.');
		}
	}
}

function convert_filters_syntax_from__Modify_HTTP_Response__to__Intercept_and_Modify_HTTP_Response(str_filter_to_be_converted){
	var str_filters = "";
	var arr_filters_formatted = [];

	try{
		str_filter_to_be_converted = str_filter_to_be_converted.trim();
		
		// All filters must have the structure [[...]]]] even if they are single HOST filters
		
		var str_filter_to_be_converted__without_spaces = str_filter_to_be_converted.replace(/\s/g, '');		
		var first_2_characters_in_filter = str_filter_to_be_converted__without_spaces.substr(0, 2);
		
		if(first_2_characters_in_filter != "[["){
			str_filter_to_be_converted = "[" + str_filter_to_be_converted + "]";
		}
		
		var arr_filters_strings = JSON.parse(str_filter_to_be_converted);
		
		for(var i=0; i<arr_filters_strings.length; i++){
			var str_filter = JSON.stringify(arr_filters_strings[i]);			
			arr_filters_formatted.push(str_filter);
		}
		
		for(var j=0; j < arr_filters_formatted.length; j++){
			var str_filters_TEMP = "";
			
			var str_filter = arr_filters_formatted[j];
			var arr_filter = JSON.parse(str_filter);
			
			var str_host = arr_filter[0];													// host
			str_host = str_host.replace(/`/g, '__backticks__');
			var str_literal__host = "`" + str_host + "`";
			
			var arr__path__search_replace = [];
			
			for(var k=1; k < arr_filter.length; k++){
				var arr_temp = arr_filter[k];
				arr__path__search_replace.push(arr_temp);
			}
			
			for(var n=0; n < arr__path__search_replace.length; n++){
				var str_path = arr__path__search_replace[n][0];								// path
				str_path = str_path.replace(/`/g, '__backticks__');
				var str_literal__path = "`" + str_path + "`";
				
				var str_literal__host_path__search_replace = "[" + str_literal__host + ", " + str_literal__path + ", ";
				
				var arr_search_replace = arr__path__search_replace[n][1];
				
				for(var m=0; m < arr_search_replace.length; m++){
					if(m % 2 == 0){											// m is even
						var str_search = arr_search_replace[m];								// search
						str_search = str_search.replace(/`/g, '__backticks__');
						var str_literal__search = "`" + str_search + "`";
						
						var str_replace = arr_search_replace[m + 1];						// replace
						str_replace = str_replace.replace(/`/g, '__backticks__');
						var str_literal__replace = "`" + str_replace + "`";
						
						var str_literal__search_replace = "[" + str_literal__search + ", " + str_literal__replace + "]";
						
						if(m < arr_search_replace.length - 2){
							str_literal__host_path__search_replace = str_literal__host_path__search_replace + str_literal__search_replace + ", ";
							
						} else if(m == arr_search_replace.length - 2){
							str_literal__host_path__search_replace = str_literal__host_path__search_replace + str_literal__search_replace + "]";
						}
					}
				}
				
				if(n < arr__path__search_replace.length - 1){
					str_filters_TEMP = str_filters_TEMP + str_literal__host_path__search_replace + ", ";
					
				} else if(n == arr__path__search_replace.length - 1){
					str_filters_TEMP = str_filters_TEMP + str_literal__host_path__search_replace;
				}
			}
			
			arr__path__search_replace.length = 0;
			
			if(j < arr_filters_formatted.length - 1){
				str_filters = str_filters + str_filters_TEMP + ", ";
				
			} else if(j == arr_filters_formatted.length - 1){
				str_filters = "[" + str_filters + str_filters_TEMP + "]";				
				str_filters = str_filters.replace(/__backticks__/g, '``');
				
				arr_filters_formatted.length = 0;
				
				return str_filters;
			}
		}
		
	} catch(err){
		// alert(err.message);
		
		arr_filters_formatted.length = 0;
		
		str_filters = "ERROR";
		
		return str_filters;
	}	
}

function reset_settings_in__prefs_js(){
	try {
		var nsIPrefBranchObj = Services.prefs.getBranch("extensions.modify_HTTP_response.");
		nsIPrefBranchObj.deleteBranch("");
		
		refresh_prefs__modify_HTTP_response();
		
		setTimeout(function() {
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("firstRunDone", true);
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("httpObserver_registered", false);
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", "");
			
			httpObserver_first_observe = false;
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "");
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", "");
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
			
			refresh_prefs__modify_HTTP_response();
			
			httpObserver_first_observe = false;
			arr_filters.length = 0;
			arr_filters_unicode.length = 0;
			
			var temp = false;
			httpObserver.unregister(temp);
	
			document.getElementById("btn_modify_HTTP_response").setAttribute("image", "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png");			
			
			close__filters_manager__and__requests_info__pages__if_opened();
			
		}, 500);
		
	} catch (err) {
		// alert(err.message);
		
		prompts.alert(null, "Reset", "ERROR : Initialization of Add-On's Settings failed.");
	}
}

function close__filters_manager__and__requests_info__pages__if_opened(){
	var filters_manager__OR__requests_info__page = check_identity_of__filters_manager_page__OR__requests_info_page();
	
	if(filters_manager__OR__requests_info__page){
		var number_of_tabs = gBrowser.tabContainer.childNodes.length;

		if(number_of_tabs == 1){
			gBrowser.contentDocument.location.href = "about:blank";
			
		} else if(number_of_tabs > 1){
			var current_tab = gBrowser.selectedTab;
			gBrowser.removeTab(current_tab);
		}
		
		prompts.alert(null, 'Reset', 'Settings have been initialized.');
		
	} else{
		prompts.alert(null, 'Reset', 'Settings have been initialized.');
	}
}

function check_identity_of__filters_manager_page__OR__requests_info_page(){
	var URL_pagina = gBrowser.contentDocument.location.href;
	var protocol = URL_pagina.split(":")[0];
	
	if(protocol == "chrome"){
		var description = "";
		
		try{
			var head_with_meta = content.document.getElementsByTagName("head")[0];
			var metas = head_with_meta.getElementsByTagName("meta");
			var description_to_check = "";

			for (i = 0; i < metas.length; i++){
				if (metas[i].name.toLowerCase() == "description") {
					description_to_check = metas[i].content.trim();
					
					if(description_to_check == "modify_HTTP_response@tellu_white"){
						description = description_to_check;
						
						break;
					}
				}
			}
			
		} catch(err){
			// alert(err.message);
			
			return false;
		}
	
		if(description == "modify_HTTP_response@tellu_white"){
			return true;
			
		} else{
			return false;
		}
		
	} else{
		return false;
	}
}

function save_filters_and_HTTP_Observer_Status_in_prefs_js(){
	var arr_filters__in_Filters_Manager = [];
	
	var arr_filter_div = content.document.getElementsByClassName("filter_div");
	
	var filter_to_save = false;

	for (var i = 0; i < arr_filter_div.length; i++){
		var arr_textarea = arr_filter_div[i].getElementsByTagName("textarea");
		
		var str_textarea_host = "";
		var str_textarea_path = "";
		var str_textarea_search = "";

		var arr_filter__in_Filters_Manager = [];
		
		for (var j = 0; j < arr_textarea.length; j++){
			if(j == 0){
				if(arr_textarea[j].value.trim() != ""){
					var host = arr_textarea[j].value;
					
					host = host.trim();
					
					arr_filter__in_Filters_Manager.push(host);						// host
					
					str_textarea_host = host;
					
				} else{
					continue;
				}
			
			} else if(j == 1){
				if(arr_textarea[j].value.trim() != ""){
					var path = arr_textarea[j].value;
					
					path = path.trim();
				
					arr_filter__in_Filters_Manager.push(path); 						// path
					
					str_textarea_path = path;
					
				} else{
					continue;
				}
			
			} else if((j > 1) && (j % 2 == 0)){				// j is even
				if(arr_textarea[j].value != ""){
					var search = arr_textarea[j].value;								// search
					var replace = arr_textarea[j+1].value;							// replace
					
					var temp = [search, replace];
					
					arr_filter__in_Filters_Manager.push(temp); 						// search + replace
					
					str_textarea_search = search;
				}
			}
		}
		
		if(str_textarea_host != "" && str_textarea_path != "" && str_textarea_search != ""){
			arr_filters__in_Filters_Manager.push(arr_filter__in_Filters_Manager);
			
			filter_to_save = true;
		}
	}

	if(filter_to_save){
		var str_filters = JSON.stringify(arr_filters__in_Filters_Manager);

		// Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_filters);
		
		var str_formated_filter_string = "";

		for(var i=0; i < str_filters.length; i++){
			str_formated_filter_string = str_formated_filter_string + format_filter_string(str_filters[i]);
		}

		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", str_formated_filter_string);
		
		httpObserver_first_observe = false;
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
		
		refresh_prefs__modify_HTTP_response();
		
		var httpObserver_registered_status = content.document.getElementById("httpObserver_registered_status").value;

		var old_favicon = content.document.getElementsByTagName("link")[0];
		var favicon = content.document.createElement("link");
		favicon.type = "image/x-icon";
		favicon.rel = "shortcut icon";
		
		if(httpObserver_registered_status == "true"){
			var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
			
			if(httpObserver_registered){
				var temp = true;
				httpObserver.unregister(temp);
			}
			
			favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_BLUE.png";
			old_favicon.parentNode.replaceChild(favicon, old_favicon);
			
			setTimeout(function() {
				httpObserver.register();
				
				prompts.alert(null, 'Save filters & HTTP Observer Status', 'The new settings have been saved in the "prefs.js" file');
				
				var number_of_tabs = gBrowser.tabContainer.childNodes.length;

				if(number_of_tabs == 1){
					gBrowser.contentDocument.location.href = "about:blank";
					
				} else if(number_of_tabs > 1){
					var current_tab = gBrowser.selectedTab;
					gBrowser.removeTab(current_tab);
				}
				
			}, 100);
			
		} else if(httpObserver_registered_status == "false"){
			favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png";
			old_favicon.parentNode.replaceChild(favicon, old_favicon);
			
			var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
			
			if(httpObserver_registered){
				var temp = false;
				httpObserver.unregister(temp);
			}
			
			prompts.alert(null, 'Save filters & HTTP Observer Status', 'The new settings have been saved in the "prefs.js" file');
			
			var number_of_tabs = gBrowser.tabContainer.childNodes.length;

			if(number_of_tabs == 1){
				gBrowser.contentDocument.location.href = "about:blank";
				
			} else if(number_of_tabs > 1){
				var current_tab = gBrowser.selectedTab;
				gBrowser.removeTab(current_tab);
			}
		}
		
	} else{
		var filter = "";

		try{
			// filter = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("filter");
			
			var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			var branch_name = "extensions.modify_HTTP_response.filter";
			var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
			filter = str.data;
			
		} catch(err){
			// alert(err.message);
		}

		if(filter != ""){
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", "");
			
			httpObserver_first_observe = false;
			
			Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");

			refresh_prefs__modify_HTTP_response();
			
			var old_favicon = content.document.getElementsByTagName("link")[0];
			var favicon = content.document.createElement("link");
			favicon.type = "image/x-icon";
			favicon.rel = "shortcut icon";
			favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png";
			old_favicon.parentNode.replaceChild(favicon, old_favicon);
			
			var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
			
			if(httpObserver_registered){
				var temp = false;
				httpObserver.unregister(temp);
			}
			
			prompts.alert(null, 'Save filters & HTTP Observer Status', 'The new settings have been saved in the "prefs.js" file');
			
			var number_of_tabs = gBrowser.tabContainer.childNodes.length;

			if(number_of_tabs == 1){
				gBrowser.contentDocument.location.href = "about:blank";
				
			} else if(number_of_tabs > 1){
				var current_tab = gBrowser.selectedTab;
				gBrowser.removeTab(current_tab);
			}
		}
	}
	
	arr_filters__in_Filters_Manager.length = 0;
}

function delete_content_of__requests_info_txt__file(){
	try{
		var data = "";
		
		var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);

		var ProfilesDir = directoryService.get("ProfD", Components.interfaces.nsIFile);
		var ProfilesDir_Path = ProfilesDir.path;
		
		var requests_info__file_path = "";

		var operating_system = get_Operating_System();

		if(operating_system == "Windows"){
			ProfilesDir_Path = ProfilesDir_Path.replace(/\\/g,'\\');
			requests_info__file_path = ProfilesDir_Path + "\\modify_HTTP_response\\requests_info.txt";
			
		} else if(operating_system == "Linux"){
			requests_info__file_path = ProfilesDir_Path + "/modify_HTTP_response/requests_info.txt";
		}

		// create an nsIFile

		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsIFile);
		file.initWithPath(requests_info__file_path);

		// file is nsIFile, data is a string

		var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

		// use 0x02 | 0x10 to open file for appending

		fileOutputStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);

		var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
		converter.init(fileOutputStream, "UTF-8", 0, 0);
		converter.writeString(data);
		fileOutputStream.close();
		converter.close();
		
		prompts.alert(null, 'Delete Content of "requests_info.txt" File', 'The Content of the "requests_info.txt" File has been Deleted');
		
	} catch(err){
		prompts.alert(null, 'Delete Content of "requests_info.txt" File', 'The Content of the "requests_info.txt" File could NOT be Deleted !');
	}
}

var manage_filters_manager_page_events = {
    handleEvent: function(event) {
		var target = event.target;
		
		var filters_manager__OR__requests_info__page = check_identity_of__filters_manager_page__OR__requests_info_page();
		
		if(filters_manager__OR__requests_info__page){
			// Prevent "filters_manager.html" OR "requests_info.html" Page to Reload with "Reload current page" Button ( id = reload-button )
			
			if (target.id == "reload-button"){
				event.stopPropagation();
				event.preventDefault();
			}
			
			// Save filters  &  HTTP Observer Status
			
			if (target.id == "btn_save_filters"){
				save_filters_and_HTTP_Observer_Status_in_prefs_js();
			}
			
			// Delete Content of "requests_info.txt" File
			
			else if(target.id == "btn_get__delete_content_of__requests_info_txt__file"){
				delete_content_of__requests_info_txt__file();
			}
			
			else if(target.id == "btn_get__All_Requested_URLs_Hosts_Paths" || target.id == "btn_get__All_HTTP_Response_for_All_URLs" || target.id == "btn_get__Received_Data_for_a_Single_Hosts_Paths"){
				// Get All Requested "URLs + Hosts + Paths"
				
				if(target.id == "btn_get__All_Requested_URLs_Hosts_Paths"){
					var url_page = "";
					
					try{
						url_page = content.document.getElementById("url_page").value;
						url_page = url_page.trim();
						
					} catch(err){}
					
					if(url_page != ""){
						var formated_url_page = "";

						for(var i=0; i < url_page.length; i++){
							formated_url_page = formated_url_page + format_filter_string(url_page[i]);
						}						
						
						var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
							
						if(modify_response_OR_get_requests_info == "modify_response"){
							var title = content.document.getElementsByTagName("title")[0];
							var favicon = content.document.createElement("link");
							favicon.type = "image/x-icon";
							favicon.rel = "shortcut icon";
							favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_PURPLE.png";				
							title.parentNode.insertBefore(favicon, title.nextSibling);
						}
						
						var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
						
						if(httpObserver_registered){
							var temp = true;
							httpObserver.unregister(temp);
						}
						
						setTimeout(function() {
							content.document.getElementById("single_host").value = "";
							content.document.getElementById("single_path").value = "";
							
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "get_info");
							
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "get_URLs_Hosts_Paths");
									
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", formated_url_page);
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
							
							refresh_prefs__modify_HTTP_response();
							
							setTimeout(function() {
								httpObserver.register();
								
								prompts.alert(null, 'Get Info', 'The new settings have been saved in the "prefs.js" file');
							
							}, 100);
							
						}, 100);
					}
				}
				
				// Get All "HTTP Responses" for All Requested URLs
				
				else if(target.id == "btn_get__All_HTTP_Response_for_All_URLs"){
					var url_page = "";
					
					try{
						url_page = content.document.getElementById("url_page").value;
						url_page = url_page.trim();
						
					} catch(err){}
					
					if(url_page != ""){
						var formated_url_page = "";

						for(var i=0; i < url_page.length; i++){
							formated_url_page = formated_url_page + format_filter_string(url_page[i]);
						}	
						
						var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
							
						if(modify_response_OR_get_requests_info == "modify_response"){
							var title = content.document.getElementsByTagName("title")[0];
							var favicon = content.document.createElement("link");
							favicon.type = "image/x-icon";
							favicon.rel = "shortcut icon";
							favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_PURPLE.png";				
							title.parentNode.insertBefore(favicon, title.nextSibling);
						}
						
						var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
						
						if(httpObserver_registered){
							var temp = true;
							httpObserver.unregister(temp);
						}
						
						setTimeout(function() {
							content.document.getElementById("single_host").value = "";
							content.document.getElementById("single_path").value = "";
							
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "get_info");
							
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "get_HTTP_Responses");
									
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", formated_url_page);
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
							
							refresh_prefs__modify_HTTP_response();
							
							setTimeout(function() {
								httpObserver.register();
								
								prompts.alert(null, 'Get Info', 'The new settings have been saved in the "prefs.js" file');
							
							}, 100);
							
						}, 100);
					}
				}
				
				// Get "Received Data" for a Single "Host + Path"
				
				else if(target.id == "btn_get__Received_Data_for_a_Single_Hosts_Paths"){
					var url_page = "";
					
					try{
						url_page = content.document.getElementById("url_page").value;
						url_page = url_page.trim();
						
					} catch(err){}
					
					var host_for_get_data = "";
					var path_for_get_data = "";
					
					try{
						host_for_get_data = content.document.getElementById("single_host").value;
						path_for_get_data = content.document.getElementById("single_path").value;
						
					} catch(err){}
					
					if(host_for_get_data.trim() != "" && path_for_get_data.trim() != ""){
						var formated_host_for_get_data = "";

						for(var i=0; i < host_for_get_data.length; i++){
							formated_host_for_get_data = formated_host_for_get_data + format_filter_string(host_for_get_data[i]);
						}
						
						var formated_path_for_get_data = "";

						for(var i=0; i < path_for_get_data.length; i++){
							formated_path_for_get_data = formated_path_for_get_data + format_filter_string(path_for_get_data[i]);
						}
						
						var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");
							
						if(modify_response_OR_get_requests_info == "modify_response"){
							var title = content.document.getElementsByTagName("title")[0];
							var favicon = content.document.createElement("link");
							favicon.type = "image/x-icon";
							favicon.rel = "shortcut icon";
							favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_PURPLE.png";				
							title.parentNode.insertBefore(favicon, title.nextSibling);
						}
						
						var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
						
						if(httpObserver_registered){
							var temp = true;
							httpObserver.unregister(temp);
						}
						
						setTimeout(function() {
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "get_info");
							
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "get_Received_Data");
									
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", url_page);
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", formated_host_for_get_data);
							Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", formated_path_for_get_data);
							
							refresh_prefs__modify_HTTP_response();
							
							setTimeout(function() {
								httpObserver.register();
								
								prompts.alert(null, 'Get Info', 'The new settings have been saved in the "prefs.js" file');
								
							}, 100);
							
						}, 100);
					}
				}
			}
			
			// Open Page with "URL = url_page"
			
			else if(target.id == "btn_get__open_page"){
				var url_page = "";
				
				try{
					var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
					var branch_name = "extensions.modify_HTTP_response.url_page";
					var str = prefManager.getComplexValue(branch_name, Components.interfaces.nsISupportsString);
					url_page = str.data;
					
				} catch (err) {
					// alert(err.message);
				}
				
				if(url_page != ""){
					gBrowser.contentDocument.location.href = url_page;
				}
			}
			
			// Clear All Settings for "get_info"
			
			else if(target.id == "btn_get__delete_settings"){
				var modify_response_OR_get_requests_info = Services.prefs.getBranch("extensions.modify_HTTP_response.").getCharPref("modify_response_OR_get_requests_info");

				if(modify_response_OR_get_requests_info == "get_info"){
					content.document.getElementById("url_page").value = "";
					content.document.getElementById("single_host").value = "";
					content.document.getElementById("single_path").value = "";
					
					var title = content.document.getElementsByTagName("title")[0];
					var favicon = content.document.createElement("link");
					favicon.type = "image/x-icon";
					favicon.rel = "shortcut icon";
					favicon.href = "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_GREY.png";				
					title.parentNode.insertBefore(favicon, title.nextSibling);
					
					var temp = false;
					httpObserver.unregister(temp);
					
					prompts.alert(null, 'Get Info', 'The new settings have been saved in the "prefs.js" file');
					
				} else if(modify_response_OR_get_requests_info == "modify_response"){
					content.document.getElementById("url_page").value = "";
					content.document.getElementById("single_host").value = "";
					content.document.getElementById("single_path").value = "";
				}
			}
			
			// Close page "filters_manager.html" OR page "requests_info.html"
			
			else if(target.id == "btn_cancel__filters_manager" || target.id == "btn_cancel__requests_info"){
				var number_of_tabs = gBrowser.tabContainer.childNodes.length;

				if(number_of_tabs == 1){
					gBrowser.contentDocument.location.href = "about:blank";
					
				} else if(number_of_tabs > 1){
					var current_tab = gBrowser.selectedTab;
					gBrowser.removeTab(current_tab);
				}
			}
		}
	}
};

window.addEventListener("click", manage_filters_manager_page_events, false);


function modify_HTTP_response__init_prefs(){
	try {
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("firstRunDone", true);
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setBoolPref("httpObserver_registered", false);
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("filter", "");
		
		httpObserver_first_observe = false;
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "");
		
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", "");
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
		Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
		
		refresh_prefs__modify_HTTP_response();
		
	} catch (err) {
		// alert(err.message);
	}
	
	// Create "requests_info.txt" file
	
	try {
		var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);

		var ProfilesDir = directoryService.get("ProfD", Components.interfaces.nsIFile);
		var ProfilesDir_Path = ProfilesDir.path;
		
		var operating_system = get_Operating_System();
		
		if(operating_system == "Windows"){
			ProfilesDir_Path = ProfilesDir_Path.replace(/\\/g,'\\');
			
		} else if(operating_system == ""){
			prompts.alert(null, "ERROR", "Unknown Operating System!\n\nThis add-on will not work properly, so it should be uninstalled.");
			
			return;
		}

		ProfilesDir.append('modify_HTTP_response');

		if(!ProfilesDir.exists()){
			// Linux Permissions = (7 << 6) | (5 << 3) | 5 /* 0755 */
			
			ProfilesDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, (7 << 6) | (5 << 3) | 5 /* 0755 */);
		}

		var requests_info__file_path = "";
		
		if(operating_system == "Windows"){
			requests_info__file_path = ProfilesDir_Path + "\\modify_HTTP_response\\requests_info.txt";
			
		} else if(operating_system == "Linux"){
			requests_info__file_path = ProfilesDir_Path + "/modify_HTTP_response/requests_info.txt";
		}

		// new file object
		var obj_requests_info_file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

		// set file with path
		obj_requests_info_file.initWithPath(requests_info__file_path);

		// if file doesn't exist, create
		if(!obj_requests_info_file.exists()) {
			obj_requests_info_file.create(0x00, 0644);
		}
		
	} catch (err) {
		// alert(err.message);
	}
}

var modify_HTTP_response__listener = {
    init: function() {
		try{
			var firstRunDone = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("firstRunDone");
			
		} catch(err){
			// alert(err.message);
		}
	  
		if (!firstRunDone) {
			modify_HTTP_response__init_prefs();
			
		} else{
			try{
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("modify_response_OR_get_requests_info", "modify_response");
				
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("get_info_option", "");
				
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("url_page", "");
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("host_for_get_data", "");
				Services.prefs.getBranch("extensions.modify_HTTP_response.").setCharPref("path_for_get_data", "");
				
				refresh_prefs__modify_HTTP_response();
				
			} catch (err) {
				// alert(err.message);
			}
			
			var filters_from__prefs_js__OK = get_filters_from__prefs_js();

			if(filters_from__prefs_js__OK){
				var httpObserver_registered = Services.prefs.getBranch("extensions.modify_HTTP_response.").getBoolPref("httpObserver_registered");
				
				if(httpObserver_registered){
					if(arr_filters.length > 0){
						httpObserver.register();
						
					} else if(arr_filters.length == 0){
						var temp = false;
						httpObserver.unregister(temp);
					}
					
				} else{
					document.getElementById("btn_modify_HTTP_response").setAttribute("image", "chrome://modify_HTTP_response/content/modify_HTTP_response_BUTTON_RED.png");
				}
				
			} else if( !filters_from__prefs_js__OK ){
				reset__modify_response__settings(filters_from__prefs_js__OK);
			}
		}
		
		window.removeEventListener("load", function() { modify_HTTP_response__listener.init() }, false);
	}
};

window.addEventListener("load", function() { modify_HTTP_response__listener.init() }, false);


var addon_listener__modify_HTTP_response = {
	onUninstalling: function(addon) {
		if (addon.id == "modify_HTTP_response@tellu_white") {
			var nsIPrefBranchObj = Services.prefs.getBranch("extensions.modify_HTTP_response.");
			nsIPrefBranchObj.deleteBranch("");
			
			// Delete "modify_HTTP_response" ( with "requests_info.txt" File )
			
			try{
				var directoryService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties); 
				var ProfilesDir = directoryService.get("ProfD", Components.interfaces.nsIFile); 
				ProfilesDir.append("modify_HTTP_response");

				ProfilesDir.remove(true);
				
			} catch(err){
				// alert(err.message);
			}
		}
	}
}

try {  
	Components.utils.import("resource://gre/modules/AddonManager.jsm");
	AddonManager.addAddonListener(addon_listener__modify_HTTP_response); 
	
} catch(err){
	// alert(err.message);
}


